/* filename:    homemh.g
** description: The program generates draws from the posterior
**              distribution via the random walk Metropolis Hastings Algorithm
**              
** created:     09/21/01
** modified:   
*/

new;
closeall;
library user, pgraph, homelib;
cls;

/******************************************************** 
**      Estimate the DSGE Model
**      Models: 1 = RBC
**              2 = HOME Production
*/ 
#include c:\projects\active\homep\Gauss\pathspec.g
lmodel = "m1";
mspec  = 2;
mprior = 1;
npara  = 18;


/******************************************************** 
**  Configure the Metropolis Algorithm
*/
mhrun   = "1";
cc0     = 1;
cc      = 0.2;
nblocks = 11;
nsim    = 10000;
opath = parapath $+ "\\";

_bounds =
    (0.1  ~ 0.99999 )|
    (0.5  ~ 0.99999 )|
    (-0.1 ~ 0.1     )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 1000    )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 100     )|
    (1E-5 ~ 100     )|
    (1E-5 ~ 100     )|
    (-10  ~ 10      )|
    (-10  ~ 10      )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     );

/******************************************************** 
** Import data on productivity growth, home investment growth,
** and hours: series (nobs,3)
** observations from 1965:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0    = 20;

/******************************************************** 
** Import the information for the prior density
*/
lprior = ftos(mprior,"%*.*lf",1,0);

priorfile = lmodel $+ lprior $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];


/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ "mode";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;
load path = ^priopath para_names;


"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);
{lnpost0, lnpy0} = fcn(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = postpath $+ "\\" $+ lmodel $+ lprior $+ "mult";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,npara );
closeall fhmult;

lhess = postpath $+ "\\" $+ lmodel $+ lprior $+ "hess";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/* Hessian is of reduced rank. Keep zero rows and columns and do SVD
*/
if mspec == 2;
   SIGPROPdim = npara;
endif;

{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun $+"\\";
ostat = opath $+ lmodel $+ lprior $+ "sta";
opara = opath $+ lmodel $+ lprior $+ "par";
statname = { POST LIKE PROP PROPPOST REJECT };

create fhstat=^ostat with ^statname, 5, 8;
create fhpara=^opara with ^para_names, npara, 8;

/******************************************************** 
**              Metropolis Hastings Algorithm
*/

/* Initialize PARANEW
*/
valid = 0;
do until valid == 1;
   paranew   = para + cc0*(sigscale*rndn(npara,1));
   {postnew, likenew} = fcn(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -1000;
      valid = 1;
   endif;
endo;   


iblock = 1;
do until iblock > nblocks;

   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1;
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
   else;
      j=1;
   endif;      

   do until j > nsim;

     paranew          = paraold + cc*(sigscale*rndn(npara,1));
     {postnew, likenew} = fcn(paranew);
     propdens         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                        -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
     propsim[j,1]     = propdens;
     proppostsim[j,1] = postnew;

     r = minc(1 | exp( postnew - postold));
     if rndu(1,1) < r;
       /* Accept proposed jump
       */
       postsim[j,1] = postnew;
       likesim[j,1] = likenew;
       parasim[j,.] = paranew';
       paraold = paranew;
       postold = postnew;
       likeold = likenew;
     else;
       /* Reject proposed jump
       */
       likesim[j,1] = likeold;
       postsim[j,1] = postold;
       parasim[j,.] = paraold';
       rej[j] = 1;
     endif;

     locate 1,1;
     "Block" iblock "of" nblocks;
     "Simulation step" j "of" nsim;
     "Rejection: r / rej / perct " r rej[j] sumc(rej)/j;
     "Likelihood     " likesim[j,1];
     "Posterior      " postsim[j,1];
     "Proposal Dens  " propsim[j,1];
     "Estimated Score" lnpy0+ln(1/(sumc(1/exp(likesim[1:j,1]-lnpy0))/j));
     "Parameters     "; parasim[j,.]; 

     j = j+1;
    endo;

    wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
    wr = writer(fhpara,parasim);

    iblock = iblock+1;
endo;  

closeall fhstat, fhpara;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para);
local lnpY, lnprio, obsmean, obsvar, lnpost;

    /* Fix parameter values here
    */
 
    if (para < _bounds[.,2]) and (para > _bounds[.,1]);

       {lnpy, obsmean, obsvar } = evalhome( para, mspec, T0 ,YY);
       lnpy    = real(lnpy);
       lnprio  = priodens( para, pmean, pstdd, pshape);
       lnpost  = lnpy + real(lnprio);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;
